@@ -13,20 +13,22 @@ Copyright:
 This code and affiliated files are (C) by Bert Constantin and individual contributors.
 Please see LICENSE and AUTHORS for more information.
 """
+from __future__ import absolute_import
 
 from django.db import models
 from django.contrib.contenttypes.models import ContentType
 from django import VERSION as django_VERSION
+from django.utils import six
 
-from base import PolymorphicModelBase
-from manager import PolymorphicManager
-from query_translate import translate_polymorphic_Q_object
+from .base import PolymorphicModelBase
+from .manager import PolymorphicManager
+from .query_translate import translate_polymorphic_Q_object
 
 
 ###################################################################################
 ### PolymorphicModel
 
-class PolymorphicModel(models.Model):
+class PolymorphicModel(six.with_metaclass(PolymorphicModelBase, models.Model)):
     """
     Abstract base class that provides polymorphic behaviour
     for any model directly or indirectly derived from it.
@@ -44,7 +46,6 @@ class PolymorphicModel(models.Model):
 
         super(YourClass,self).save(*args,**kwargs)
     """
-    __metaclass__ = PolymorphicModelBase
 
     # for PolymorphicModelBase, so it can tell which models are polymorphic and which are not (duck typing)
     polymorphic_model_marker = True
@@ -94,20 +95,34 @@ class PolymorphicModel(models.Model):
         return super(PolymorphicModel, self).save(*args, **kwargs)
 
     def get_real_instance_class(self):
-        """Normally not needed.
+        """
+        Normally not needed.
         If a non-polymorphic manager (like base_objects) has been used to
         retrieve objects, then the real class/type of these objects may be
-        determined using this method."""
+        determined using this method.
+        """
         # the following line would be the easiest way to do this, but it produces sql queries
-        #return self.polymorphic_ctype.model_class()
-        # so we use the following version, which uses the CopntentType manager cache
-        return ContentType.objects.get_for_id(self.polymorphic_ctype_id).model_class()
+        # return self.polymorphic_ctype.model_class()
+        # so we use the following version, which uses the CopntentType manager cache.
+        # Note that model_class() can return None for stale content types;
+        # when the content type record still exists but no longer refers to an existing model.
+        try:
+            return ContentType.objects.get_for_id(self.polymorphic_ctype_id).model_class()
+        except AttributeError:
+            # Django <1.6 workaround
+            return None
 
     def get_real_concrete_instance_class_id(self):
-        return ContentType.objects.get_for_model(self.get_real_instance_class(), for_concrete_model=True).pk
+        model_class = self.get_real_instance_class()
+        if model_class is None:
+            return None
+        return ContentType.objects.get_for_model(model_class, for_concrete_model=True).pk
 
     def get_real_concrete_instance_class(self):
-        return ContentType.objects.get_for_model(self.get_real_instance_class(), for_concrete_model=True).model_class()
+        model_class = self.get_real_instance_class()
+        if model_class is None:
+            return None
+        return ContentType.objects.get_for_model(model_class, for_concrete_model=True).model_class()
 
     def get_real_instance(self):
         """Normally not needed.
@@ -154,7 +169,7 @@ class PolymorphicModel(models.Model):
         subclasses_and_superclasses_accessors = self._get_inheritance_relation_fields_and_models()
 
         from django.db.models.fields.related import SingleRelatedObjectDescriptor, ReverseSingleRelatedObjectDescriptor
-        for name, model in subclasses_and_superclasses_accessors.iteritems():
+        for name, model in subclasses_and_superclasses_accessors.items():
             orig_accessor = getattr(self.__class__, name, None)
             if type(orig_accessor) in [SingleRelatedObjectDescriptor, ReverseSingleRelatedObjectDescriptor]:
                 #print >>sys.stderr, '---------- replacing', name, orig_accessor, '->', model
